import sqlite3
import os
from pathlib import Path
import logging

def get_config_dir()->Path:
    return Path('C:/ProgramData/Incubator')

def get_db_path()->Path:
    return get_config_dir().joinpath("Incubator.db")

def create_db_dir():
    config_path = get_config_dir()
    try:
        if not config_path.exists():
            os.mkdir(config_path)
            logging.info(("Created Directory:", config_path))
    except Exception:
        logging.error((f"Could not create", config_path))


def create_db():
    config_path = get_config_dir()
    db_path = get_db_path()
    try:
        if not config_path.exists():
            create_db_dir()

        if db_path.exists():
            return

        #connetct to db/create empty db
        conn = sqlite3.connect(db_path)
        cursor = conn.cursor()

        #Create table PlateActions
        cursor.execute("""  CREATE TABLE "PlateActions" (
                            "Timestamp" TEXT NOT NULL CONSTRAINT "PK_PlateActions" PRIMARY KEY,
                            "PlateId" TEXT NULL,
                            "Action" TEXT NULL,
                            CONSTRAINT "FK_PlateActions_Plates_PlateId" FOREIGN KEY ("PlateId") REFERENCES "Plates" ("PlateId") ON DELETE RESTRICT)""")

        #create table Plates
        cursor.execute("""  CREATE TABLE "Plates" (
                            "PlateId" TEXT NOT NULL CONSTRAINT "PK_Plates" PRIMARY KEY,
                            "Owner" TEXT NULL,
                            "Location" INTEGER NOT NULL)""")

        #create table slots
        cursor.execute("""  CREATE TABLE "Slots" (
                            "SlotNumber" INTEGER NOT NULL,
                            "Stacker" INTEGER NOT NULL,
                            "Role" INTEGER NOT NULL,
                            "Occupied" INTEGER NOT NULL,
                            "AssignedPlatePlateId" TEXT NULL,
                            "CurrentPlateId" TEXT NULL,
                            CONSTRAINT "PK_Slots" PRIMARY KEY ("Stacker", "SlotNumber"),
                            CONSTRAINT "FK_Slots_Plates_AssignedPlatePlateId" FOREIGN KEY ("AssignedPlatePlateId") REFERENCES "Plates" ("PlateId") ON DELETE RESTRICT,
                            CONSTRAINT "FK_Slots_Plates_CurrentPlateId" FOREIGN KEY ("CurrentPlateId") REFERENCES "Plates" ("PlateId") ON DELETE RESTRICT) """)

        #create index IX_PlateActions_PlateId
        cursor.execute("""  CREATE INDEX "IX_PlateActions_PlateId" ON "PlateActions" ("PlateId")""")

        #create Index Slots_AssignedPlatePlateId
        cursor.execute("""  CREATE UNIQUE INDEX "IX_Slots_AssignedPlatePlateId" ON "Slots" ("AssignedPlatePlateId")""")

        #create Index Slots_CurrentPlateID
        cursor.execute("""  CREATE INDEX "IX_Slots_CurrentPlateId" ON "Slots" ("CurrentPlateId")""")

        conn.close()

    except sqlite3.Error as e:
        logging.error(msg =("Sqlite Error occurred", e))

create_db()


import ttkbootstrap as ttk
from ttkbootstrap.constants import *
from src.incubator_db.db import DB

class GUI:
    def __init__(self):
        self.root = ttk.Window(title = "Incubator Data Base", themename="superhero")
        self.root.geometry("1200x800")
        self.db = DB()
        self.tables = self.db.get_tables()

        self.root.columnconfigure(0, weight=1)
        self.root.columnconfigure(1, weight=1)
        self.root.columnconfigure(2, weight=1)
        self.root.rowconfigure(0, weight=0)
        self.root.rowconfigure(1, weight=8)

        style = ttk.Style()
        style.configure("Default.TButton")

        self.bool_slot = False
        self.bool_plate_actions = False
        self.bool_plates = False

        self.button_slots = ttk.Button(text="Slots", style="Default.TButton", command=self.callback_slots)
        self.button_plates = ttk.Button(text="Plates", style="Default.TButton", command = self.callback_plates)
        self.button_plate_actions = ttk.Button(text="PlateActions", style="Default.TButton", command = self.callback_plate_actions)

        self.button_slots.grid(column=0, row=0, sticky='nsew', ipady=12)
        self.button_plates.grid(column=1, row= 0, sticky='nsew', ipady=12)
        self.button_plate_actions.grid(column=2, row=0, sticky='nsew', ipady=12)

        self.tree_plates = ttk.Treeview(self.root)
        self.tree_plate_actions = ttk.Treeview(self.root)
        self.tree_slots = ttk.Treeview(self.root)

        self.init_tree_plates()
        self.init_tree_plate_actions()
        self.init_tree_slots()
        self.insert_data()
        self.current_tree = self.tree_plates
        self.current_tree.grid()


    def get_root(self)->ttk.Window:
        return self.root

    def init_tree_plates(self):
        columns = self.db.get_columns_from_table(self.tables[1])
        self.tree_plates["columns"] = columns

        self.tree_plates.column("#0", width=0, stretch=False)
        self.tree_plates.heading("#0", text="")

        for i in columns:
            self.tree_plates.column(i, anchor=CENTER)
            self.tree_plates.heading(i, text=i, anchor=CENTER)

        self.tree_plates.grid(row=1, column=0, columnspan=3, sticky='nsew', pady=5)
        self.tree_plates.grid_remove()

    def init_tree_slots(self):
        columns = self.db.get_columns_from_table(self.tables[2])
        self.tree_slots["columns"] = columns

        self.tree_slots.column("#0", width=0, stretch=False)
        self.tree_slots.heading("#0", text="")

        for i in columns:
            self.tree_slots.column(i, anchor=CENTER)
            self.tree_slots.heading(i, text = i, anchor=CENTER)

        self.tree_slots.grid(row=1, column=0, columnspan=3, sticky='nsew', pady=5)
        self.tree_slots.grid_remove()

    def init_tree_plate_actions(self):
        columns = self.db.get_columns_from_table(self.tables[0])
        self.tree_plate_actions["columns"] = columns

        self.tree_plate_actions.column("#0", width=0, stretch=False)
        self.tree_plate_actions.heading("#0", text="")

        for i in columns:
            self.tree_plate_actions.column(i, anchor=CENTER)
            self.tree_plate_actions.heading(i, text=i, anchor=CENTER)

        self.tree_plate_actions.grid(row=1, column=0, columnspan=3, sticky='nsew', pady=5)
        self.tree_plate_actions.grid_remove()


    def insert_data(self):
        table_data_plate_action: list[tuple] = self.db.get_table_data(self.tables[0])
        table_data_plates: list[tuple] = self.db.get_table_data(self.tables[1])
        table_data_slots: list[tuple] = self.db.get_table_data(self.tables[2])

        for i in range(len(table_data_plate_action)):
            self.tree_plates.insert(parent="", index = 'end', iid=i, text="Parent", values=table_data_plate_action[i])

        for i in range(len(table_data_plates)):
            self.tree_plates.insert(parent="", index = 'end', iid=i, text="Parent", values=table_data_plates[i])

        for i in range(len(table_data_slots)):
            self.tree_plates.insert(parent="", index = 'end', iid=i, text="Parent", values=table_data_slots[i])

    def callback_slots(self):
        if self.bool_slot:
            return

        self.bool_plates = False
        self.bool_plate_actions = False
        self.bool_slot = True
        self.current_tree.grid_remove()
        self.current_tree = self.tree_slots
        self.current_tree.grid()

    def callback_plates(self):
        if self.bool_plates:
            return

        self.bool_slot = False
        self.bool_plate_actions = False
        self.bool_plates = True
        self.current_tree.grid_remove()
        self.current_tree = self.tree_plates
        self.current_tree.grid()

    def callback_plate_actions(self):
        if self.bool_plate_actions:
            return

        self.bool_plates = False
        self.bool_slot = False
        self.bool_plate_actions = True
        self.current_tree.grid_remove()
        self.current_tree = self.tree_plate_actions
        self.current_tree.grid()